from enum import Enum

class TimePeriods(Enum):
    QuarterHourly = "quarterhourly"
    Hourly = "hourly"
    Daily = "daily"
    Weekly = "weekly"
    BiWeekly = "biweekly"
    Monthly = "monthly"
    Yearly = "yearly"
    UnSet = "unset"


from enum import Enum

class DatePartModelType(Enum):
    GreaterOrEqual = "gteq"
    LessOrEqual = "lteq"
    In = "in"
    Equal = "eq"
    Less = "lt"
    Greater = "gt"
    Not = "not"
    Unset = ""


from datetime import datetime, date, time
import pytest
from ..models.locale.enums.querytype import QueryType
from ..services.locale.querytypes.const import QUERYTYPE_AVERAGEOFTHREEDAYS, QUERYTYPE_AVERAGEOFTHREEHOURS, QUERYTYPE_SOLLENTUNA
from ..services.locale.querytypes.querytypes import QUERYTYPES
from ..services.locale.countries.sweden import SE_SHE_AB, SE_Bjerke_Energi, SE_Gothenburg, SE_Kristinehamn, SE_Skovde, SE_Sollentuna, SE_Ellevio,SE_JBF
from ..services.locale.countries.belgium import VregBelgium
from ..services.locale.countries.default import NoPeak

def test_SE_Bjerke_Energi():
    p = SE_Bjerke_Energi
    assert p.free_charge(p, mockdt=datetime.combine(date(2005, 7, 14), time(22, 30))) is True
    assert p.free_charge(p, mockdt=datetime.combine(date(2005, 7, 14), time(15,00))) is False
    del(p)

def test_generic_querytype_avg_threedays():
    pt = QUERYTYPES[QueryType.AverageOfThreeDays]
    pt.reset()
    pt.try_update(new_val=1.2, timestamp=datetime.combine(date(2022, 7, 14), time(20, 30)))
    pt.try_update(new_val=2, timestamp=datetime.combine(date(2022, 7, 14), time(21, 30)))
    to_state_machine = pt.peaks.export_peaks
    pt.peaks.set_init_dict(dict_data=to_state_machine, dt=datetime.combine(date(2022, 7, 14), time(23, 30)))
    pt.try_update(new_val=0.6, timestamp=datetime.combine(date(2022, 7, 15), time(21, 30)))
    assert len(pt.peaks._p) == 2
    assert pt._charged_peak_value == 1.3

def test_generic_querytype_avg_threedays2():
    pg = QUERYTYPES[QueryType.AverageOfThreeDays]
    pg.reset()
    pg.try_update(new_val=1.2, timestamp=datetime.combine(date(2022, 7, 14), time(20, 30)))
    pg.try_update(new_val=2, timestamp=datetime.combine(date(2022, 7, 14), time(21, 30)))
    assert len(pg.peaks._p) == 1
    assert pg._charged_peak_value == 2

def test_generic_querytype_avg_threedays3():
    to_state_machine = {'m': 7, 'p': {'14h21': 2}}
    p1 = QUERYTYPES[QueryType.AverageOfThreeDays]
    p1.reset()
    p1.try_update(new_val=1, timestamp=datetime.combine(date(2022, 7, 15), time(21, 30)))
    p1.peaks.set_init_dict(to_state_machine, datetime.combine(date(2022, 7, 15), time(21, 30)))
    assert len(p1.peaks._p) == 2
    assert p1.charged_peak == 1.5
    assert p1.observed_peak == 1
    p1.try_update(new_val=2, timestamp=datetime.combine(date(2022, 7, 15), time(22, 30)))
    assert len(p1.peaks._p) == 2
    assert p1.charged_peak == 2
    assert p1.observed_peak == 2

def test_faulty_number_in_import():
    to_state_machine = {'m': 7, 'p': {'14h21': 2, '11h22': 1.49, '12h9': 1.93, '12h14': 0.73}}
    p1 = QUERYTYPES[QueryType.AverageOfThreeDays]
    p1.reset()
    p1.try_update(new_val=1, timestamp=datetime.combine(date(2022, 7, 15), time(21, 30)))
    p1.peaks.set_init_dict(to_state_machine, datetime.combine(date(2022, 7, 15), time(21, 30)))
    assert len(p1.peaks._p) == 3
    assert p1.charged_peak == 1.81
    assert p1.observed_peak == 1.49
    p1.try_update(new_val=1.5, timestamp=datetime.combine(date(2022, 7, 15), time(22, 30)))
    assert len(p1.peaks._p) == 3
    assert p1.charged_peak == 1.81
    assert p1.observed_peak == 1.5
    
def test_overridden_number_in_import():
    to_state_machine = {'m': 7, 'p': {'11h22': 1.49, '12h9': 1.93, '13h16': 0.86}}
    p1 = QUERYTYPES[QueryType.AverageOfThreeDays]
    p1.reset()
    p1.try_update(new_val=0.22, timestamp=datetime.combine(date(2022, 7, 13), time(21, 30)))
    p1.peaks.set_init_dict(to_state_machine, datetime.combine(date(2022, 7, 13), time(21, 30)))
    assert p1.charged_peak == 1.43

def test_SE_Gothenburg():
    p = SE_Gothenburg
    assert p.free_charge(p) is False
    p.query_model.try_update(new_val=1.2, timestamp=datetime.combine(date(2022, 7, 14), time(22, 30)))
    p.query_model.try_update(new_val=1, timestamp=datetime.combine(date(2022, 7, 16), time(22, 30)))
    p.query_model.try_update(new_val=1.5, timestamp=datetime.combine(date(2022, 7, 17), time(22, 30)))
    p.query_model.try_update(new_val=1.7, timestamp=datetime.combine(date(2022, 7, 17), time(22, 30)))
    p.query_model.try_update(new_val=1.5, timestamp=datetime.combine(date(2022, 7, 19), time(22, 30)))
    assert p.query_model.observed_peak > 0
    del(p)

def test_generic_querytype_avg_threehour2s():
    p = SE_Sollentuna
    p.query_model.try_update(new_val=1.2, timestamp=datetime.combine(date(2022, 7, 14), time(22, 30)))
    p.query_model.try_update(new_val=1, timestamp=datetime.combine(date(2022, 7, 16), time(22, 30)))
    p.query_model.try_update(new_val=1.5, timestamp=datetime.combine(date(2022, 7, 17), time(22, 30)))
    p.query_model.try_update(new_val=1.7, timestamp=datetime.combine(date(2022, 7, 17), time(22, 30)))
    p.query_model.try_update(new_val=1.5, timestamp=datetime.combine(date(2022, 7, 19), time(22, 30)))
    assert p.query_model.observed_peak == 0
    d1 = date(2022, 7, 14)
    t = time(22, 30)
    dt1 = datetime.combine(d1, t)
    p.query_model.try_update(new_val=1.2, timestamp=dt1)
    d2 = date(2022, 7, 16)
    dt2 = datetime.combine(d2, t)
    p.query_model.try_update(new_val=1, timestamp=dt2)
    d3 = date(2022, 7, 17)
    dt3 = datetime.combine(d3, t)
    p.query_model.try_update(new_val=1.5, timestamp=dt3)
    d3 = date(2022, 7, 17)
    dt3 = datetime.combine(d3, t)
    p.query_model.try_update(new_val=1.7, timestamp=dt3)
    d4 = date(2022, 7, 19)
    dt4 = datetime.combine(d4, t)
    p.query_model.try_update(new_val=1.5, timestamp=dt4)
    
# def test_SE_Kristinehamn():
#     p = SE_Kristinehamn
#     p.query_model.try_update(new_val=0.5, timestamp=datetime.combine(date(2023, 6, 14), time(20, 30)))
#     assert p.query_model.charged_peak == 0.5
#     p.query_model.try_update(new_val=1.2, timestamp=datetime.combine(date(2023, 2, 14), time(16, 30)))
#     assert p.query_model.charged_peak == 1.2

def test_peak_new_month():
    p = SE_Gothenburg
    p.query_model.try_update(new_val=1.2, timestamp=datetime.combine(date(2022, 6, 2), time(22, 30)))
    p.query_model.try_update(new_val=1, timestamp=datetime.combine(date(2022, 6, 16), time(22, 30)))
    p.query_model.try_update(new_val=1.5, timestamp=datetime.combine(date(2022, 6, 17), time(20, 30)))
    p.query_model.try_update(new_val=1.7, timestamp=datetime.combine(date(2022, 6, 17), time(22, 30)))
    p.query_model.try_update(new_val=1.5, timestamp=datetime.combine(date(2022, 6, 19), time(22, 30)))
    assert len(p.query_model.peaks._p) == 3
    assert p.query_model.observed_peak == 1.2
    p.query_model.try_update(new_val=0.03, timestamp=datetime.combine(date(2022, 7, 1), time(0, 0)))
    assert len(p.query_model.peaks._p) == 1
    assert p.query_model.observed_peak == 0.03
    
def test_peak_new_hour():
    p = SE_Gothenburg
    p.query_model.try_update(new_val=1.2, timestamp=datetime.combine(date(2022, 6, 1), time(1, 30)))
    assert p.query_model.peaks.p == {(1, 1): 1.2}
    p.query_model.try_update(new_val=1, timestamp=datetime.combine(date(2022, 6, 1), time(6, 30)))
    assert p.query_model.peaks.p == {(1, 1): 1.2}
    p.query_model.try_update(new_val=1.5, timestamp=datetime.combine(date(2022, 6, 1), time(9, 30)))
    assert p.query_model.peaks.p == {(1, 9): 1.5}

def test_peak_new_hour_multiple():
    p = SE_Gothenburg
    p.query_model.try_update(new_val=1.2, timestamp=datetime.combine(date(2022, 7, 2), time(22, 30)))
    p.query_model.try_update(new_val=1, timestamp=datetime.combine(date(2022, 7, 16), time(22, 30)))
    p.query_model.try_update(new_val=1.5, timestamp=datetime.combine(date(2022, 7, 17), time(20, 30)))
    p.query_model.try_update(new_val=1.7, timestamp=datetime.combine(date(2022, 7, 17), time(22, 30)))
    p.query_model.try_update(new_val=1.5, timestamp=datetime.combine(date(2022, 7, 19), time(22, 30)))
    assert p.query_model.peaks.export_peaks == {'m': 7, 'p': {'2h22': 1.2, '17h22': 1.7, '19h22': 1.5}}
    assert p.query_model.peaks.p == {(2,22): 1.2, (17,22): 1.7, (19,22): 1.5}
    assert p.query_model.peaks.m == 7
    p.query_model.try_update(new_val=2.5, timestamp=datetime.combine(date(2022, 7, 19), time(23, 30)))
    assert p.query_model.peaks.export_peaks == {'m': 7, 'p': {'2h22': 1.2, '17h22': 1.7, '19h23': 2.5}}
    assert p.query_model.peaks.p == {(2,22): 1.2, (17,22): 1.7, (19,23): 2.5}
    
def test_overridden_number_in_import():
    to_state_machine = {'m': 7, 'p': {'1h15': 1.5}}
    p1 = QUERYTYPES[QueryType.AverageOfThreeDays]
    p1.reset()
    p1.try_update(new_val=0.22, timestamp=datetime.combine(date(2022, 7, 2), time(15, 30)))
    p1.peaks.set_init_dict(to_state_machine, datetime.combine(date(2022, 7, 2), time(15, 30)))
    assert len(p1.peaks._p) == 2

def test_quarterly():
    p = SE_Kristinehamn
    assert not p.is_quarterly(p)
    p2 = VregBelgium
    assert p2.is_quarterly(p2)

def test_peak_new_month_2():
    p = SE_Gothenburg
    p.query_model.try_update(new_val=1.2, timestamp=datetime.combine(date(2022, 7, 2), time(22, 30)))
    p.query_model.try_update(new_val=1, timestamp=datetime.combine(date(2022, 7, 16), time(22, 30)))
    p.query_model.try_update(new_val=1.5, timestamp=datetime.combine(date(2022, 7, 17), time(20, 30)))
    p.query_model.try_update(new_val=1.7, timestamp=datetime.combine(date(2022, 7, 17), time(22, 30)))
    p.query_model.try_update(new_val=1.5, timestamp=datetime.combine(date(2022, 7, 19), time(22, 30)))
    assert len(p.query_model.peaks._p) == 3
    assert p.query_model.observed_peak == 1.2
    p.query_model.try_update(new_val=0.03, timestamp=datetime.combine(date(2022, 8, 1), time(22, 30)))
    assert p.query_model.observed_peak == 0.03
    assert len(p.query_model.peaks._p) == 1
    p.query_model.try_update(new_val=0.06, timestamp=datetime.combine(date(2022, 8, 2), time(22, 30)))
    assert len(p.query_model.peaks._p) == 2
    assert p.query_model.charged_peak == 0.04
    assert p.query_model.observed_peak == 0.03

def test_se_ellevio():
    p = SE_Ellevio
    assert p.free_charge(p, mockdt=datetime.now()) is False
    p.query_model.try_update(new_val=1.2, timestamp=datetime.combine(date(2022, 7, 14), time(22, 30)))
    p.query_model.try_update(new_val=1, timestamp=datetime.combine(date(2022, 7, 16), time(22, 30)))
    p.query_model.try_update(new_val=1.5, timestamp=datetime.combine(date(2022, 7, 17), time(22, 30)))
    p.query_model.try_update(new_val=1.7, timestamp=datetime.combine(date(2022, 7, 17), time(22, 30)))
    p.query_model.try_update(new_val=1.5, timestamp=datetime.combine(date(2022, 7, 19), time(22, 30)))
    assert p.query_model.observed_peak > 0
    del(p)

def test_se_jbf():
    p = SE_JBF
    assert p.free_charge(p, mockdt=datetime.combine(date(2023, 2, 14), time(21, 59))) is False
    assert p.free_charge(p, mockdt=datetime.combine(date(2023, 2, 14), time(22, 1))) is True
    assert p.free_charge(p, mockdt=datetime.combine(date(2023, 5, 17), time(12, 0))) is True

    p.query_model.try_update(new_val=1.2, timestamp=datetime.combine(date(2023, 2, 6), time(10, 30)))
    p.query_model.try_update(new_val=1, timestamp=datetime.combine(date(2023, 2, 6), time(11, 30)))
    assert p.query_model.observed_peak == 1
    assert p.query_model.charged_peak == 1.1
    p.query_model.try_update(new_val=2, timestamp=datetime.combine(date(2023, 2, 6), time(12, 30)))
    assert p.query_model.charged_peak == 1.4
    del(p)    

# def test_no_peak():
#     p = NoPeak
#     assert p.free_charge(p, mockdt=datetime.now()) is True
#     p.query_model.try_update(new_val=1.5, timestamp=datetime.combine(date(2023, 7, 19), time(22, 30)))
#     assert p.query_model.charged_peak == 0


from datetime import datetime
from typing import List
from ....models.locale.enums.dividents import Dividents
from ....models.locale.enums.datepart_modeltype import DatePartModelType
from ....models.locale.enums.datepart_datetype import DatePartDateType
from ....models.locale.queryservice_model import QueryServiceModel

class QueryService:
    def __init__(self, args: QueryServiceModel=QueryServiceModel()):
        self._settings = args
    
    def should_register_peak(self, dt: datetime) -> bool:
        main_ret = []
        main_grouping = (s for s in self._settings.groups if s.divident is not Dividents.UNSET)
        for s in main_grouping:
            group_ret = []
            grouping = (a for a in s.dateparts if len(a.values))
            for a in grouping:
                group_ret.append(QueryService.datepart(a.type, a.dttype, a.values, dt))
            if s.divident is Dividents.AND:
                main_ret.append(all(group_ret))
            else:
                main_ret.append(any(group_ret))
        return any(main_ret) if len(main_ret) else True

    @staticmethod
    def datepart(logic: str, dtpart: str, arguments: List[int], timer: datetime) -> bool:
        if not arguments:
            return True
        arg = arguments if len(arguments) > 1 else arguments[0]
        _logic = QueryService.LOGIC.get(logic)(
            QueryService.DATETIMEPARTS.get(dtpart)(timer), 
            arg
            )
        return _logic

    AND = "AND"
    OR = "OR"
    LOGIC = {
        DatePartModelType.Equal: lambda a, dtp : dtp == a,
        DatePartModelType.Less: lambda a, dtp : a < dtp,
        DatePartModelType.Greater: lambda a, dtp : a > dtp,
        DatePartModelType.Not: lambda a, dtp : dtp != a,
        DatePartModelType.LessOrEqual: lambda a, dtp : a <= dtp,
        DatePartModelType.GreaterOrEqual: lambda a, dtp : a >= dtp,
        DatePartModelType.In: lambda a, dtp : a in dtp
    }
    DATETIMEPARTS = {
        DatePartDateType.Weekday: lambda d : d.weekday(),
        DatePartDateType.Month:  lambda d : d.month,
        DatePartDateType.Hour:  lambda d : d.hour,
    }













from dataclasses import dataclass, field
from typing import List

@dataclass
class Price:
    price_aware: bool = False
    min_price: float = 0.0
    top_price: float = 0.0
    cautionhour_type: str = ""
    dynamic_top_price: bool = False

@dataclass
class Charger:
    chargertype: str = ""
    chargerid: str = ""
    powerswitch: str = ""
    powermeter: str = ""

@dataclass
class HubOptions:
    locale: str = field(init=False)
    charger: Charger = Charger()
    price: Price = Price()
    peaqev_lite: bool = False
    powersensor_includes_car: bool = False
    powersensor: str = field(init=False)
    startpeaks: dict = field(default_factory=dict)
    cautionhours: List = field(default_factory=lambda: [])
    nonhours: List = field(default_factory=lambda: [])
    fuse_type: str = ""
    blocknocturnal: bool = False
    """blocks charging during the night for safety concern reasons."""




# import requests
# import json
# from dataclasses import dataclass

# @dataclass
# class StatsModel:
#     identifier: str
#     version: str
#     type: int
#     chargertype: str
#     locale: str
#     priceaware: bool

# class Api:
#     api_url = "https://jsonplaceholder.typicode.com/todos/1"
#     headers =  {"Content-Type":"application/json"}

#     def post(self, _postdata: StatsModel) -> None:        
#         requests.post(self.api_url, data=json.dumps(_postdata), headers=self.headers)



# from unittest import mock
# import pytest
# import time
# from ..services.chargecontroller.chargecontrollerbase import ChargeControllerBase
# from ..models.chargecontroller_states import ChargeControllerStates

# _CHARGERSTATESMOCK = {
#     ChargeControllerStates.Idle: ["idle"],
#     ChargeControllerStates.Connected: ["connected"],
#     ChargeControllerStates.Charging: ["charging"],
# }
# _NONHOURSMOCK = [5,6,7]

# # def test_init_chargecontrollerbase_ok():
# #     c = ChargeControllerBase(
# #         charger_state_translation=_CHARGERSTATESMOCK,
# #         non_hours=_NONHOURSMOCK
# #     )

# #     print(c.latest_charger_start)
# #     print(time.time())
# #     time.sleep(0.1)
# #     assert c.latest_charger_start < time.time()

# # def test_init_chargecontrollerbase_init_fail_empty_statesdict():
# #     with pytest.raises(AssertionError):
# #         ChargeControllerBase(
# #         charger_state_translation={},
# #         non_hours=_NONHOURSMOCK
# #     )

# # def test_init_chargecontrollerbase_init_fail_empty_statesdict_type():
# #     with pytest.raises(AssertionError):
# #         ChargeControllerBase(
# #         charger_state_translation={CHARGERSTATES.Idle: []},
# #         non_hours=_NONHOURSMOCK
# #     )

# # def test_done_timeout_override():
# #     mocktimeout = 60
# #     c = ChargeControllerBase(
# #         charger_state_translation=_CHARGERSTATESMOCK,
# #         non_hours=_NONHOURSMOCK,
# #         timeout= mocktimeout
# #     )

# #     assert c.done_timeout == mocktimeout

# # def test_is_timeout_true():
# #     mocktimeout = 2
# #     c = ChargeControllerBase(
# #         charger_state_translation=_CHARGERSTATESMOCK,
# #         non_hours=_NONHOURSMOCK,
# #         timeout= mocktimeout
# #     )
# #     time.sleep(mocktimeout)
    
# #     assert c._is_timeout == True

# # def test_is_timeout_false():
# #     mocktimeout = 2
# #     c = ChargeControllerBase(
# #         charger_state_translation=_CHARGERSTATESMOCK,
# #         non_hours=_NONHOURSMOCK,
# #         timeout= mocktimeout
# #     )

# #     assert c._is_timeout == False

# def test_start_threshold_should_start():
#     c = ChargeControllerBase(
#         charger_state_translation=_CHARGERSTATESMOCK,
#         non_hours=_NONHOURSMOCK
#     )

#     ret = c._below_start_threshold(
#         predicted_energy=0.2,
#         current_peak=1,
#         threshold_start=0.6
#     )
#     assert ret is True

# def test_start_threshold_should_not_start():
#     c = ChargeControllerBase(
#         charger_state_translation=_CHARGERSTATESMOCK,
#         non_hours=_NONHOURSMOCK
#     )
    
#     ret = c._below_start_threshold(
#         predicted_energy=0.8,
#         current_peak=1,
#         threshold_start=0.6
#     )
#     assert ret is False

# def test_stop_threshold_should_stop():
#     c = ChargeControllerBase(
#             charger_state_translation=_CHARGERSTATESMOCK,
#             non_hours=_NONHOURSMOCK
#         )
#     ret = c._above_stop_threshold(
#         predicted_energy=0.8,
#         current_peak=1,
#         threshold_stop=0.6
#     )
#     assert ret is True

# def test_stop_threshold_should_not_stop():
#     c = ChargeControllerBase(
#         charger_state_translation=_CHARGERSTATESMOCK,
#         non_hours=_NONHOURSMOCK
#     )
    
#     ret = c._above_stop_threshold(
#         predicted_energy=0.2,
#         current_peak=1,
#         threshold_stop=0.6
#     )
#     assert ret is False

# def test_stop_threshold_should_not_stop():
#     c = ChargeControllerBase(
#         charger_state_translation=_CHARGERSTATESMOCK,
#         non_hours=_NONHOURSMOCK
#     )
    
#     ret = c._above_stop_threshold(
#         predicted_energy=2,
#         current_peak=1.98,
#         threshold_stop=60
#     )
#     assert ret is False

# # def test_chargecontroller_is_charging_nonhour():
# #     c = ChargeControllerBase(
# #         charger_state_translation=_CHARGERSTATESMOCK,
# #         non_hours=_NONHOURSMOCK
# #     )
    
# #     ret = c.get_status(
# #         charger_state="charging",
# #         charger_enabled=True,
# #         charger_done=False,
# #         car_power_sensor=50,
# #         total_energy_this_hour=0.5,
# #         current_hour=6    
# #     )
# #     assert ret == CHARGERSTATES.Stop

# # def test_chargecontroller_is_idle():
# #     c = ChargeControllerBase(
# #         charger_state_translation=_CHARGERSTATESMOCK,
# #         non_hours=_NONHOURSMOCK
# #     )
    
# #     ret = c.get_status(
# #         charger_state="idle",
# #         charger_enabled=False,
# #         charger_done=False,
# #         car_power_sensor=50,
# #         total_energy_this_hour=0.5,
# #         current_hour=10    
# #     )
# #     assert ret == CHARGERSTATES.Idle

# # def test_chargecontroller_is_connected():
# #     c = ChargeControllerBase(
# #         charger_state_translation=_CHARGERSTATESMOCK,
# #         non_hours=_NONHOURSMOCK
# #     )
    
# #     ret = c.get_status(
# #         charger_state="connected",
# #         charger_enabled=False,
# #         charger_done=False,
# #         car_power_sensor=50,
# #         total_energy_this_hour=0.5,
# #         current_hour=10    
# #     )
# #     assert ret == CHARGERSTATES.Connected

# # def test_chargecontroller_is_idle_nonhour():
# #     c = ChargeControllerBase(
# #         charger_state_translation=_CHARGERSTATESMOCK,
# #         non_hours=_NONHOURSMOCK
# #     )
    
# #     ret = c.get_status(
# #         charger_state="idle",
# #         charger_enabled=False,
# #         charger_done=False,
# #         car_power_sensor=50,
# #         total_energy_this_hour=0.5,
# #         current_hour=6    
# #     )
# #     assert ret == CHARGERSTATES.Idle

# # def test_chargecontroller_is_connected_nonhour():
# #     c = ChargeControllerBase(
# #         charger_state_translation=_CHARGERSTATESMOCK,
# #         non_hours=_NONHOURSMOCK
# #     )
    
# #     ret = c.get_status(
# #         charger_state="connected",
# #         charger_enabled=False,
# #         charger_done=False,
# #         car_power_sensor=50,
# #         total_energy_this_hour=0.5,
# #         current_hour=6    
# #     )
# #     assert ret == CHARGERSTATES.Connected


import logging
from .hoursbase import Hours
from ..hoursselection import Hoursselection as core_hours
from ....models.hourselection.cautionhourtype import CautionHourType

_LOGGER = logging.getLogger(__name__)

class PriceAwareHours(Hours):
    def __init__(
            self,
            hub
    ):
        self._hub = hub
        self._cautionhour_type = CautionHourType.get_num_value(hub.options.price.cautionhour_type)
        self._cautionhour_type_string = hub.options.price.cautionhour_type
        self._core = core_hours(
            absolute_top_price=self._set_absolute_top_price(hub.options.price.top_price),
            min_price=hub.options.price.min_price,
            cautionhour_type=self._cautionhour_type_string, 
            blocknocturnal=hub.options.blocknocturnal
        )
        self._hass = hub.state_machine
        self._prices = []
        super().__init__(price_aware=True)

    @property
    def options(self):
        return self._core.model.options

    @property
    def dynamic_caution_hours(self) -> dict:
        return self._core.dynamic_caution_hours

    @property
    def cautionhour_type_string(self) -> str:
        return self._cautionhour_type_string

    @property
    def non_hours(self) -> list:
        return self._core.non_hours

    @property
    def caution_hours(self) -> list:
        return self._core.caution_hours

    @property
    def absolute_top_price(self):
        return self._core.options.absolute_top_price

    @property
    def min_price(self):
        return self._core.options.min_price

    @property
    def prices(self) -> list:
        return self._core.prices

    @prices.setter
    def prices(self, val):
        self._core.prices = val

    @property
    def prices_tomorrow(self) -> list:
        return self._core.prices_tomorrow

    @prices_tomorrow.setter
    def prices_tomorrow(self, val):
        self._core.prices_tomorrow = val

    @property
    def adjusted_average(self) -> float:
        return self._core.adjusted_average

    @adjusted_average.setter
    def adjusted_average(self, val):
        if isinstance(val, (float, int)):
            self._core.adjusted_average = val

    @property
    def offsets(self) -> dict:
        return self._core.offsets

    @property
    def is_initialized(self) -> bool:
        if self.prices is not None:
            if len(self.prices):
                if self._is_initialized is False:
                    self._is_initialized = True
                    _LOGGER.debug("Hourselection has initialized")
                return True
        return False

    def update_top_price(self, dyn_top_price) -> None: 
        if self._hub.options.price.dynamic_top_price:
            self._core.update_top_price(dyn_top_price)

    def get_average_kwh_price(self):
        if self._is_initialized:
            try:
                return self._core.get_average_kwh_price()
            except ZeroDivisionError as e:
                _LOGGER.warning(f"get_average_kwh_price could not be calculated: {e}")
            return 0
        _LOGGER.debug("get avg kwh price, not initialized")
        return "-"

    def get_total_charge(self):
        if self._is_initialized:
            try:
                return self._core.get_total_charge(self._hub.sensors.current_peak.value)
            except ZeroDivisionError as e:
                _LOGGER.warning(f"get_total_charge could not be calculated: {e}")
            return 0
        _LOGGER.debug("get avg kwh price, not initialized")
        return "-"

    @staticmethod
    def _set_absolute_top_price(_input) -> float:
        if _input is None or _input <= 0:
            return float("inf")
        return _input


import logging
from datetime import datetime
from typing import Tuple
import statistics as stat

from .hoursselection_helpers import create_dict
from .hourselection_calculations import normalize_prices, rank_prices, get_offset_dict, should_be_cautionhour, set_charge_allowance
from ....models.hourselection.hourobjects.hourobject import HourObject
from ....models.hourselection.hourobjects.hourobject_helpers import update_interim_lists
from ....models.hourselection.hourselectionmodels import HourSelectionModel

from ....models.hourselection.hourtypelist import HourTypeList

_LOGGER = logging.getLogger(__name__)

import time

class HourselectionService2:
    def __init__(self, parent, base_mock_hour: int = None):
        self._latest_update = time.time()
        self.parent = parent
        self._mock_hour = base_mock_hour
        self._preserve_interim: bool = False

    def get_prices(self):
        pass

    def rank_prices(self, prices):
        pass

    def check_boundaries(self):
        #min max prices
        #cautiontype
        #cautiontype-maxhrs?
        pass

    def set_models(self):
        #non
        #caution
        #offsets
        pass



class HourSelectionService:
    def __init__(self, parent, base_mock_hour: int = None):
        self.parent = parent
        self._mock_hour = base_mock_hour
        self._preserve_interim: bool = False

    def update(self, caller: str = None) -> None:
        if self._preserve_interim and caller == "today":
            self.parent.model.hours.hours_today = self.parent.model.hours.hours_tomorrow
            self.parent.model.hours.hours_tomorrow = HourObject([], [], {})
            return

        today=self._update_per_day(prices=self.parent.model.prices_today)
        tomorrow=self._update_per_day(prices=self.parent.model.prices_tomorrow)
        
        hours, hours_tomorrow = self._interim_day_update(today, tomorrow)

        self.parent.model.hours.hours_today = self._add_remove_limited_hours(hours)
        self.parent.model.hours.hours_tomorrow = self._add_remove_limited_hours(hours_tomorrow)
        self.update_hour_lists()

    def _update_per_day(self, prices: list, range_start: int =0) -> HourObject:
        pricedict = {}
        if prices is not None and len(prices) > 1:
            pricedict = create_dict(prices)
            normalized_pricedict = create_dict(
                normalize_prices(prices)
                )
            if stat.stdev(prices) > 0.05:
                ranked = rank_prices(
                        pricedict, 
                        normalized_pricedict,
                        self.parent.cautionhour_type_enum,
                        range_start,
                        self.parent.model.adjusted_average,
                        self.parent.model.options.blocknocturnal
                        )
                ready_hours = self._determine_hours(ranked, prices)
                
                ret= HourObject(
                    nh=ready_hours.nh, 
                    ch=ready_hours.ch, 
                    dyn_ch=ready_hours.dyn_ch, 
                    pricedict=pricedict
                    )
            else:
                """the price curve is too flat to determine hours"""
                ret= HourObject(nh=[], ch=[], dyn_ch={},pricedict=pricedict)
            ret.offset_dict=get_offset_dict(normalized_pricedict)
            return ret
        return HourObject([],[],{})

    def update_hour_lists(self, listtype:HourTypeList = None) -> None:
        hour = self.set_hour()
        match listtype:
            case HourTypeList.NonHour:
                self.parent.model.hours.update_non_hours(hour)
            case HourTypeList.CautionHour:
                self.parent.model.hours.update_caution_hours(hour)
            case HourTypeList.DynCautionHour:
                self.parent.model.hours.update_dynanmic_caution_hours(hour)   
            case _:
                self.parent.model.hours.update_all(hour)

    def _add_remove_limited_hours(self, hours: HourObject) -> HourObject:
        """Removes cheap hours and adds expensive hours set by user limitation"""
        if hours is None or all(
            [
                len(hours.nh) == 0, 
                len(hours.ch) == 0, 
                len(hours.dyn_ch) == 0
            ]):
            return HourObject(
                offset_dict=hours.offset_dict,
                pricedict=hours.pricedict
                )
        hours.add_expensive_hours(self.parent.model.options.absolute_top_price)
        hours.remove_cheap_hours(self.parent.model.options.min_price)
        
        return hours

    def _determine_hours(self, price_list: dict, prices: list) -> HourObject:
        ret = HourObject([],[],{})
        ch_type = self.parent.model.options.cautionhour_type
        peak = self.parent.model.current_peak
        try:
            for p in price_list:    
                if price_list[p]["force_non"] is True:
                    ret.nh.append(p)
                elif should_be_cautionhour(price_list[p], prices, peak, ch_type):
                    ret.ch.append(p)
                    ret.dyn_ch[p] = round(set_charge_allowance(price_list[p]["permax"], ch_type),2)
                else:
                    ret.nh.append(p)
        except IndexError as e:
            _LOGGER.error(f"Error in determine hours: {e}")
        return ret
    
    def set_hour(self, testhour:int = None) -> int:
        return testhour if testhour is not None else self._mock_hour if self._mock_hour is not None else datetime.now().hour

    def _interim_day_update(self, today: HourObject, tomorrow: HourObject) -> Tuple[HourObject, HourObject]:
        """Updates the non- and caution-hours with an adjusted mean of 14h - 13h today-tomorrow to get a more sane nightly curve."""
        if len(self.parent.model.prices_tomorrow) < 23:
            return today, tomorrow 
        #hour = self._mock_hour if self._mock_hour is not None else 14
        hour =14
        negative_hour = (24 - hour)*-1
        pricelist = self.parent.model.prices_today[hour::]
        pricelist[len(pricelist):] = self.parent.model.prices_tomorrow[0:hour]
        new_hours = self._update_per_day(pricelist, hour)
        #print(f"new: {new_hours.nh}")
        today = update_interim_lists(range(hour,24), today, new_hours, hour)
        tomorrow = update_interim_lists(range(0,hour), tomorrow, new_hours, negative_hour)

        self._preserve_interim = True
        return today, tomorrow



from ..services.prediction.prediction import Prediction as p
from ..PeaqErrors import PeaqValueError
import pytest

def test_prediction():
       ret = p._predicted_energy(
       now_min=13,
       now_sec=37,
       power_avg=420,
       total_hourly_energy=0.24
       )

       assert ret == 0.565

def test_prediction_percentage():
       ret = p._predicted_energy(
       now_min=13,
       now_sec=37,
       power_avg=420,
       total_hourly_energy=0.24
       )
       retperc = p._predicted_percentage_of_peak(2, ret)

       assert retperc == 28.25

def test_prediction_minute_overflow():
       with pytest.raises(ValueError):
              p._predicted_energy(
              now_min=60,
              now_sec=37,
              power_avg=420,
              total_hourly_energy=0.24)

def test_prediction_second_overflow():
       with pytest.raises(ValueError):
              p._predicted_energy(
              now_min=50,
              now_sec=60,
              power_avg=420,
              total_hourly_energy=0.24)

def test_prediction_hour_overflow():
       with pytest.raises(ValueError):
              p._predicted_energy(
              now_min=-5,
              now_sec=37,
              power_avg=420,
              total_hourly_energy=0.24)

def test_prediction_second_negative():
       with pytest.raises(ValueError):
              p._predicted_energy(
              now_min=50,
              now_sec=-2,
              power_avg=420,
              total_hourly_energy=0.24)

# def test_prediction_hourlyenergy_negative():
#        with pytest.raises(ValueError):
#               p._predicted_energy(
#               now_min=50,
#               now_sec=4,
#               power_avg=420,
#               total_hourly_energy=-0.24)


def test_prediction_quarterly():
       ret = p._predicted_energy(
       now_min=13,
       now_sec=37,
       power_avg=420,
       total_hourly_energy=0.24,
       is_quarterly=True
       )

       ret2 = p._predicted_energy(
       now_min=28,
       now_sec=37,
       power_avg=420,
       total_hourly_energy=0.24,
       is_quarterly=True
       )

       ret3 = p._predicted_energy(
       now_min=28,
       now_sec=0,
       power_avg=420,
       total_hourly_energy=0.24,
       is_quarterly=True
       )

       assert ret == ret2
       assert ret < ret3

def test_prediction_percentage_neg_poweravg():
       ret = p._predicted_energy(
       now_min=13,
       now_sec=37,
       power_avg=-420,
       total_hourly_energy=0.24
       )
       retperc = p._predicted_percentage_of_peak(2, ret)

       assert retperc >= 0

def test_prediction_percentage_neg_energy():
       ret = p._predicted_energy(
       now_min=13,
       now_sec=37,
       power_avg=-420,
       total_hourly_energy=-0.24
       )
       retperc = p._predicted_percentage_of_peak(2, ret)

       assert retperc >= 0

def test_prediction_percentage_neg_energy_and_poweravg():
       ret = p._predicted_energy(
       now_min=13,
       now_sec=37,
       power_avg=-420,
       total_hourly_energy=-0.24
       )
       retperc = p._predicted_percentage_of_peak(2, ret)

       assert retperc >= 0

def test_prediction():
       ret = p._predicted_energy(
       now_min=48,
       now_sec=0,
       power_avg=0,
       total_hourly_energy=0.25
       )

       assert ret == 0.25


from ....services.locale.querytypes.queryservice import QueryService
from ....models.locale.enums.querytype import QueryType
from ..time_pattern import TimePattern
from ....models.locale.price.locale_price import LocalePrice
from ....models.locale.price.models.seasoned_price import SeasonedPrice
from ....models.locale.enums.price_type import PriceType
from ....models.locale.enums.calendar_periods import CalendarPeriods
from ..querytypes.const import (
    QUERYTYPE_BASICMAX_MON_FRI_07_17_DEC_MAR_ELSE_REGULAR,
    QUERYTYPE_SOLLENTUNA,
    QUERYTYPE_MAX_NOV_MAR_MON_FRI_06_22,
    QUERYTYPE_AVERAGEOFTHREEHOURS_MON_FRI_07_19,
)

from ..querytypes.querysets import QUERYSETS
from ..querytypes.querytypes import QUERYTYPES
from dataclasses import dataclass
from ..locale_model import Locale_Type


@dataclass(frozen=True)
class SE_Sollentuna(Locale_Type):
    observed_peak = QueryType.AverageOfThreeHours
    charged_peak = QueryType.AverageOfThreeHours
    query_model = QUERYTYPES[QueryType.AverageOfThreeHours]
    query_model.set_query_service(QueryService(QUERYSETS[QUERYTYPE_SOLLENTUNA]))

    free_charge_pattern = TimePattern([
        {
            CalendarPeriods.Month:   [*range(1,13)],
            CalendarPeriods.Weekday: [0, 1, 2, 3, 4],
            CalendarPeriods.Hour:    [19, 20, 21, 22, 23, 0, 1, 2, 3, 4, 5, 6]
        },
        {
            CalendarPeriods.Month:   [*range(1,13)],
            CalendarPeriods.Weekday: [5, 6],
            CalendarPeriods.Hour:    [*range(0,24)]
        }
    ])

    # Rörlig avgift sommar april – oktober 61,46 kr/kW
    # Rörlig avgift vinter november – mars 122,92 kr/kW
    # https://www.seom.se/el/elnat/2022-ars-priser-och-villkor/


@dataclass(frozen=True)
class SE_Skovde(Locale_Type):
    observed_peak = QueryType.Max
    charged_peak = QueryType.Max
    query_model = QUERYTYPES[QueryType.Max]
    query_model.set_query_service(QueryService(QUERYSETS[QUERYTYPE_MAX_NOV_MAR_MON_FRI_06_22]))

    free_charge_pattern = TimePattern([
        {
            CalendarPeriods.Month:   [11, 12, 1, 2, 3],
            CalendarPeriods.Weekday: [5, 6],
            CalendarPeriods.Hour:    [22, 23, 0, 1, 2, 3, 4, 5]
        },
        {
            CalendarPeriods.Month:   [4, 5, 6, 7, 8, 9, 10],
            CalendarPeriods.Weekday: [*range(0,7)],
            CalendarPeriods.Hour:    [*range(0,24)]
        }
    ])

    # November-Mars, vardagar (mån-fre) 06-22
    # single peak i denna period månadsvis.


@dataclass(frozen=True)
class SE_SHE_AB(Locale_Type):
    observed_peak = QueryType.AverageOfThreeHours
    charged_peak = QueryType.AverageOfThreeHours
    query_model = QUERYTYPES[QueryType.AverageOfThreeHours]
    query_model.set_query_service(QueryService(QUERYSETS[QUERYTYPE_AVERAGEOFTHREEHOURS_MON_FRI_07_19]))

    free_charge_pattern = TimePattern([
        {
            CalendarPeriods.Month:   [*range(1,13)],
            CalendarPeriods.Weekday: [5, 6],
            CalendarPeriods.Hour:    [19, 20, 21, 22, 23, 0, 1, 2, 3, 4, 5, 6]
        }
    ])

    """
    Elnätskunder med effekttaxa får vinterpris på överföringsavgift från och med 1 november – 31 mars. 
    Prishöjningen på effekttaxan skedde 1 april men blir mer kännbart när vinterpriset nu träder i kraft. 
    Kunder som bor i villa och har effekttaxa kan påverka sin kostnad genom att försöka skjuta på sådan förbrukning 
    som är möjlig från dagtid till kvällstid (19.00-07:00) eller till helger och röda dagar då det är helt kostnadsfritt att använda elnätet.
    Överföringsavgiften beräknas på medelmånadsvärdet av de tre högsta effektvärden dagtid vardagar mellan 07.00-19.00.
    
    Nov – Mars vardagar kl 7-19 135,00 kr/kW inkl moms
    April – Okt vardagar kl 7-19 56,00 kr/kW inkl moms
    """


@dataclass(frozen=True)
class SE_Partille(Locale_Type):
    observed_peak = QueryType.Max
    charged_peak = QueryType.Max
    query_model = QUERYTYPES[QueryType.Max]


@dataclass(frozen=True)
class SE_FalbygdensEnergi(Locale_Type):
    observed_peak = QueryType.Max
    charged_peak = QueryType.Max
    query_model = QUERYTYPES[QueryType.Max]


@dataclass(frozen=True)
class SE_Nacka_normal(Locale_Type):
    observed_peak = QueryType.AverageOfThreeHours
    charged_peak = QueryType.AverageOfThreeHours
    query_model = QUERYTYPES[QueryType.AverageOfThreeHours]

    # https://www.nackaenergi.se/images/downloads/natavgifter/FAQ_NYA_TARIFFER.pdf


@dataclass(frozen=True)
class SE_NACKA_timediff(Locale_Type):
    pass

    # this class is for nacka time differentiated peaks.
    # https://www.nackaenergi.se/images/downloads/natavgifter/FAQ_NYA_TARIFFER.pdf


@dataclass(frozen=True)
class SE_Malung_Salen(Locale_Type):
    observed_peak = QueryType.AverageOfFiveDays
    charged_peak = QueryType.AverageOfFiveDays
    query_model = QUERYTYPES[QueryType.AverageOfFiveDays]
    free_charge_pattern = TimePattern([
        {
            CalendarPeriods.Month:   [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12],
            CalendarPeriods.Weekday: [0, 1, 2, 3, 4, 5, 6],
            CalendarPeriods.Hour:    [19, 20, 21, 22, 23, 0, 1, 2, 3, 4, 5, 6]
        }
    ])

    # Rörlig avgift sommar april – oktober 35 kr/kW
    # Rörlig avgift vinter november – mars 118,75 kr/kW


@dataclass(frozen=True)
class SE_Linde_Energi(Locale_Type):
    observed_peak = QueryType.Max
    charged_peak = QueryType.Max
    query_model = QUERYTYPES[QueryType.Max]
    # docs: https://www.lindeenergi.se/elnat/elnatspriser/effekttariffer.4.1491a0b016e44ba6ccfe91b4.html

    """
    Din effektavgift baseras på din högsta effekttopp per månad, alltså den timme per månad då du använder mest el sammantaget, oavsett tid på dygnet. 
    Under perioden november till mars tillkommer en högbelastningslastavgift, så kallad höglasteffekt, då du även debiteras för ditt högst uppmätta timvärde vardagar kl 07:00-19:00.
    Under vintermånaderna kan du alltså debiteras för två olika timvärden, beroende på när på dygnet din effekttopp uppmäts.
    """


@dataclass(frozen=True)
class SE_Kristinehamn(Locale_Type):
    observed_peak = QueryType.Max
    charged_peak = QueryType.Max
    query_model = QUERYTYPES[QueryType.Max]
    query_model.set_query_service(QueryService(QUERYSETS[QUERYTYPE_BASICMAX_MON_FRI_07_17_DEC_MAR_ELSE_REGULAR]))

    """
    https://kristinehamnsenergi.se/elnat/elnatsavgiften/effektavgift-villa-med-bergvarmepump/
    vardagar november-mars, kl 07.00-17.00 > highload instead of normal load. other times, normal load
    """


@dataclass(frozen=True)
class SE_Karlstad(Locale_Type):
    observed_peak = QueryType.Max  # todo check if correct
    charged_peak = QueryType.Max  # todo check if correct
    query_model = QUERYTYPES[QueryType.Max]
    # docs: https://karlstadsnat.se/elnat/kund/priser-och-tariffer/effekttariff/
    """
    Note, high load extra is added on weekdays from 6-18 during november - march. 
    This does not affect the peak, but should in future updates be cause for forced non-/or caution-hours to lessen the cost for the consumer.
    """


@dataclass(frozen=True)
class SE_Gothenburg(Locale_Type):
    observed_peak = QueryType.AverageOfThreeDays
    charged_peak = QueryType.AverageOfThreeDays
    query_model = QUERYTYPES[QueryType.AverageOfThreeDays]
    price = LocalePrice(
        price_type=PriceType.Static,
        value=36.25,
        currency="SEK"
    )

@dataclass(frozen=True)
class SE_Ellevio(Locale_Type):
    #docs: https://www.ellevio.se/privat/om-din-el/forsta-din-elkostnad/ny-prismodell-for-elnatet/
    observed_peak = QueryType.AverageOfThreeDays
    charged_peak = QueryType.AverageOfThreeDays
    query_model = QUERYTYPES[QueryType.AverageOfThreeDays]
    price = LocalePrice(
        price_type=PriceType.Static,
        value=65,
        currency="SEK"
    )

@dataclass(frozen=True)
class SE_JBF(Locale_Type):
    #docs: http://www.jbf.nu/sv/natavgift
    observed_peak = QueryType.AverageOfThreeHours
    charged_peak = QueryType.AverageOfThreeHours
    query_model = QUERYTYPES[QueryType.AverageOfThreeHours]
    query_model.set_query_service(QueryService(QUERYSETS[QUERYTYPE_MAX_NOV_MAR_MON_FRI_06_22]))
    price = LocalePrice(
        price_type=PriceType.Static,
        value=96.6,
        currency="SEK"
    )
    free_charge_pattern = TimePattern([
        {
            CalendarPeriods.Month:   [4,5,6,7,8,9,10],
            CalendarPeriods.Weekday: [*range(0,7)],
            CalendarPeriods.Hour:    [*range(0,24)]
        },
        {
            CalendarPeriods.Month:   [11,12,1,2,3],
            CalendarPeriods.Weekday: [5,6],
            CalendarPeriods.Hour:    [*range(0,24)]
        },
        {
            CalendarPeriods.Month:   [11,12,1,2,3],
            CalendarPeriods.Weekday: [0,1,2,3,4],
            CalendarPeriods.Hour:    [22,23,0,1,2,3,4,5]
        }
    ])
    #Högbelastningstid är vardagar kl. 06.00 – 22.00 under månaderna november – mars.

@dataclass(frozen=True)
class SE_Bjerke_Energi(Locale_Type):
    observed_peak = QueryType.Max
    charged_peak = QueryType.Max
    query_model = QUERYTYPES[QueryType.Max]
    price = LocalePrice(
        price_type=PriceType.Seasoned,
        currency="SEK",
        value=[
        SeasonedPrice(validity=TimePattern([
        {
            CalendarPeriods.Month:   [11,12,1,2,3],
            CalendarPeriods.Weekday: [0, 1, 2, 3, 4, 5, 6],
            CalendarPeriods.Hour:    [6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21]
        }
    ]), 
    value=106.25), 
    SeasonedPrice(validity=TimePattern([
        {
            CalendarPeriods.Month:   [4, 5, 6, 7, 8, 9, 10],
            CalendarPeriods.Weekday: [0, 1, 2, 3, 4, 5, 6],
            CalendarPeriods.Hour:    [6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21]
        }
    ]), 
    value=50)
    ])
    
    free_charge_pattern = TimePattern([
        {
            CalendarPeriods.Month:   [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12],
            CalendarPeriods.Weekday: [0, 1, 2, 3, 4, 5, 6],
            CalendarPeriods.Hour:    [22, 23, 0, 1, 2, 3, 4, 5]
        }
    ])

    # docs: https://www.bjerke-energi.se/elnat/tariffer/effekttariff-fr-o-m-2022-02-01/
    # dag kl. 06-22 nov-mars                 106,25 kr/kW/mån
    # dag kl. 06-22 april-okt                50 kr/kW/mån
    # natt kl. 22-06 alla dagar hela året    0 kr/kW/mån
    """
    Note, high load extra is added from 06-22 during november - march. 
    This does not affect the peak, but should in future 
    """


@dataclass(frozen=True)
class SE_Telge_Energi(Locale_Type):
    observed_peak = QueryType.Max
    charged_peak = QueryType.Max
    query_model = QUERYTYPES[QueryType.Max]
    free_charge_pattern = TimePattern([
        {
            CalendarPeriods.Month:   [4, 5, 6, 7, 8, 9, 10],
            CalendarPeriods.Weekday: [0, 1, 2, 3, 4, 5, 6],
            CalendarPeriods.Hour:    [21, 22, 23, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
                                      20]
        }
        ,
        {
            CalendarPeriods.Month:   [11, 12, 1, 2, 3],
            CalendarPeriods.Weekday: [5, 6],
            CalendarPeriods.Hour:    [21, 22, 23, 0, 1, 2, 3, 4, 5, 6]
        }
    ]
    )

    # docs: https://nya.telge.se/elnat/ny-prismodell-elnat/om-hog--och-laglasttid/

    """
    Det högre priset gäller bara på helgfria vardagar 07.00-20.00 under november till mars.
    """


@dataclass(frozen=True)
class SE_Malarenergi(Locale_Type):
    observed_peak = QueryType.AverageOfThreeHours
    charged_peak = QueryType.AverageOfThreeHours
    query_model = QUERYTYPES[QueryType.AverageOfThreeHours]

    # docs: https://www.malarenergi.se/el/elnat/effekttariffer/

    """
    Det högre priset gäller bara på helgfria vardagar 07.00-20.00 under november till mars.
    """


@dataclass(frozen=True)
class SE_TekniskaVerken_Link(Locale_Type):
    observed_peak = QueryType.Max
    charged_peak = QueryType.Max
    query_model = QUERYTYPES[QueryType.Max]

    # docs: unknown


@dataclass(frozen=True)
class SE_Eskilstuna(Locale_Type):
    observed_peak = QueryType.Max
    charged_peak = QueryType.Max
    query_model = QUERYTYPES[QueryType.Max]

    # docs: unknown
